#include "fileobj.h"
#include "mfinderapp.h"
#include "functions.h"

char cutname[256] = "";
extern char appsPath[];
extern bool dropopenapp;

FileObj::FileObj(QWidget *parent, const char *path, 
										const char *name, int type, int viewtype)
        :QWidget(parent, path),
      	QDropSite(this)
{
	char *pref, *suff, *token, *delims = ".";
	char fname[MAX_NAME_LEN] = "";
	FILE *fAppData;
	
	strcpy(myPath, path);
	strcpy(myName, name);
	if(strcmp(myPath, "/") != 0)
		sprintf(myFullPath, "%s/%s", myPath, myName);
	else
		sprintf(myFullPath, "%s%s", myPath, myName);
	myType = type;
	this->viewtype = viewtype;
	if(myType == DR || myType == DRLNK)	
		isDir = TRUE;
	else
		isDir = FALSE;
	if(myType == FILLNK || myType == DRLNK || myType == FILEXELNK)	
		readlink(myFullPath, myLinkTo, MAX_PATH_LEN);

	isApp = FALSE;

	if(myType == FIL)	{
		strcpy(myEditor, ((mfinderapp *)qApp)->editorOfFileAtPath(myFullPath));

		if(strcmp(myPath, appsPath) == 0) {
			sprintf(fname, "%s", myName);
			token = strtok(fname, delims);
			pref = token;
			while(token != NULL) {
				suff = token;
				token = strtok(NULL, delims);
			}
			if(strcmp(suff, "app") == 0) {
				isApp = TRUE;
				fAppData = fopen(myFullPath, "r");
				fscanf(fAppData, "%s", myAppName);
				fclose(fAppData);
			}
		}
	}
		
	myobjsframe = (ObjsFrame *)parent;
	
	QColor color = QColor(230, 230, 230);
	QColorGroup cnorm = QColorGroup(black, color, color.light(), color.dark(), 
																color.dark(120), black, color);
	QColorGroup cinv = QColorGroup(white, black, color.light(), color.dark(), 
															color.dark(120), white, black);
	pnorm = QPalette(cnorm, cnorm, cnorm);
	pinv = QPalette(cinv, cinv, cinv);
	font = QFont("Helvetica", 10);

	havecustomicon = ((mfinderapp *)qApp)->hasCustomIconFileAtPath(myFullPath);

	if(viewtype == BYICONS) 
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
															 FALSE, UNSELECT, havecustomicon));
	else
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
		 														TRUE, UNSELECT, havecustomicon));

	resize(icona.width(), icona.height());

	label = new QLabel(myobjsframe, myName);
	label->setAutoResize(TRUE);
	label->setPalette(pnorm);
	label->setFont(font);
	label->installEventFilter(this); 
	if(viewtype == BYICONS) 
		label->setText(cutFileLabelText(myName));
	else
		label->setText(myName);
	
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isOpenLocal = FALSE;
	mustdrawselrect = FALSE;
	popupMenu = NULL;
	dragdelay = 0;
	r1o = QRect(0,0,0,0);
	r2o = QRect(0,0,0,0);
	r1v = QRect(0,0,0,0);
	r2v = QRect(0,0,0,0);
	
	if(isDir)
		if(((mfinderapp *)qApp)->winAtPath(myFullPath))
			UnselOpen(true);

	DeskObj *d = ((mfinderapp *)qApp)->deskObjectAtPath(myFullPath);
	if(d) {
		setMyDeskObj(d);
		d->setMyRefObj(this);
	} else {
		ismydeskobj = FALSE;
	}
	
	show();
	label->show();
}

FileObj::~FileObj()
{
	if(ismydeskobj)
		mydeskobj->unsetMyRefObj();
	label->~QLabel();
}

Popup* FileObj::createPopup()
{
	Popup *popup;
	
	popup = new Popup();
	popup->insertItem("Open", this, SLOT(openFromPopUp()));
	popup->insertItem("Rename", this, SLOT(renameFromPopUp()));
	popup->insertItem("Info", this, SLOT(infoFromPopUp()));
	if(myType != DR && myType != DRLNK) {
		popup->insertItem("Edit", this, SLOT(editFromPopUp()));
		popup->insertItem("Open With...", this, SLOT(openWithFromPopUp()));
	}
	popup->insertItem("Duplicate", this, SLOT(duplicateFromPopUp()));
	popup->insertItem("Delete", this, SLOT(deleteFromPopUp()));
   return popup;
}

void FileObj::InfoObj()
{
	struct dirent **namelist;
	struct stat listStat;
	struct passwd *usr;
	struct group *grp;
	struct tm *t;
	mode_t oldperm = 0L, newperm = 0L;
	char user[32] = "", group[32] = "";
	char oldusr[32] = "", oldgroup[32] = "";
	int m, ln;
	char ERR[256];
    
	lstat(myFullPath, &listStat);
	
	if((usr = getpwuid(listStat.st_uid)) == NULL)
		sprintf(user, "%i", listStat.st_uid);
	else
		sprintf(user, "%s", usr->pw_name);
	if((grp = getgrgid(listStat.st_gid)) == NULL) {
		sprintf(group, "%i", listStat.st_gid);
	} else {
		QString grname = grp->gr_name;
		if(grname.isEmpty()) 
			sprintf(group, "%i", grp->gr_gid);
		else	
			sprintf(group, "%s", grp->gr_name);
	}
					
	IWin = new InfoWin(0L, myFullPath, myName, myType, havecustomicon, TRUE);

	if(myType == FILLNK || myType == FILEXELNK || myType == DRLNK) {
		sprintf(IWin->fSize, "Link to: %s", myLinkTo);
	} else if(myType == FIL || myType == FILEXE) {
		sprintf(IWin->fSize, "Size: %i", (int)listStat.st_size);
	} else if(myType == DR) {
		int m = scandir(myFullPath, &namelist, 0, alphasort);
		sprintf(IWin->fSize, "Elements: %i", m - 2);
	} else {
		sprintf(IWin->fSize, " ");
	}
	
	t = localtime(&listStat.st_mtime);
	sprintf(IWin->lastMod, "Last modification: %02i:%02i %02i.%02i.%04i", 
			t->tm_hour, t->tm_min, t->tm_mday, t->tm_mon + 1, t->tm_year + 1900);

	bool IamRoot = (geteuid() == 0);
	bool IsMyFile = (geteuid() == listStat.st_uid);
	IWin->userEdit->setText(user);
	sprintf(oldusr, "%s", IWin->userEdit->text());
	IWin->userEdit->setEnabled(IamRoot || IsMyFile);
	IWin->groupEdit->setText(group);
	sprintf(oldgroup, "%s", IWin->groupEdit->text());
	IWin->groupEdit->setEnabled(IamRoot || IsMyFile);

	IWin->ckUread->setChecked((listStat.st_mode & S_IRUSR) == S_IRUSR);
	IWin->ckUread->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IRUSR);
	IWin->ckUwrite->setChecked((listStat.st_mode & S_IWUSR) == S_IWUSR);
	IWin->ckUwrite->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IWUSR);
	IWin->ckUexec->setChecked((listStat.st_mode & S_IXUSR) == S_IXUSR);
	IWin->ckUexec->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IXUSR);
	IWin->ckUID->setChecked((listStat.st_mode & S_ISUID) == S_ISUID);
	IWin->ckUID->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_ISUID);

	IWin->ckGread->setChecked((listStat.st_mode & S_IRGRP) == S_IRGRP);
	IWin->ckGread->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IRGRP);
	IWin->ckGwrite->setChecked((listStat.st_mode & S_IWGRP) == S_IWGRP);
	IWin->ckGwrite->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IWGRP);
	IWin->ckGexec->setChecked((listStat.st_mode & S_IXGRP) == S_IXGRP);
	IWin->ckGexec->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IXGRP);
	IWin->ckGID->setChecked((listStat.st_mode & S_ISGID) == S_ISGID);
	IWin->ckGID->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_ISGID);

	IWin->ckOread->setChecked((listStat.st_mode & S_IROTH) == S_IROTH);
	IWin->ckOread->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IROTH);
	IWin->ckOwrite->setChecked((listStat.st_mode & S_IWOTH) == S_IWOTH);
	IWin->ckOwrite->setEnabled(IsMyFile || IamRoot);	
	oldperm |= (listStat.st_mode & S_IWOTH);
	IWin->ckOexec->setChecked((listStat.st_mode & S_IXOTH) == S_IXOTH);
	IWin->ckOexec->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IXOTH);
	IWin->ckSticky->setChecked((listStat.st_mode & S_ISVTX) == S_ISVTX);
	IWin->ckSticky->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_ISVTX);

	if(myType == FIL)
		IWin->neweditorfield->setText(((mfinderapp *)qApp)->editorOfFileAtPath(myFullPath));
		
	if(IWin->exec()) {
		if(IWin->ckUread->isChecked())
		   newperm |= S_IRUSR;
		if(IWin->ckUwrite->isChecked())
		   newperm |= S_IWUSR;
		if(IWin->ckUexec->isChecked())
		   newperm |= S_IXUSR;
		if(IWin->ckUID->isChecked())
		   newperm |= S_ISUID;

		if(IWin->ckGread->isChecked())
		   newperm |= S_IRGRP;
		if(IWin->ckGwrite->isChecked())
		   newperm |= S_IWGRP;
		if(IWin->ckGexec->isChecked())
		   newperm |= S_IXGRP;
		if(IWin->ckGID->isChecked())
		   newperm |= S_ISGID;

		if(IWin->ckOread->isChecked())
		   newperm |= S_IROTH;
		if(IWin->ckOwrite->isChecked())
		   newperm |= S_IWOTH;
		if(IWin->ckOexec->isChecked())
		   newperm |= S_IXOTH;
		if(IWin->ckSticky->isChecked())
		   newperm |= S_ISVTX;

		if(oldperm != newperm) {
			if(chmod(myFullPath, newperm)) {
				sprintf(ERR, "Could not change permissions\n");
				if(dialog != 0L)
            	delete dialog;
				dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
				m = dialog->exec();						
			} else {
				updateMyData();
			}
		}
		if(strcmp(oldusr, IWin->userEdit->text()) != 0 ||
									strcmp(oldgroup, IWin->groupEdit->text()) != 0) {

			usr = getpwnam(IWin->userEdit->text());
			grp = getgrnam(IWin->groupEdit->text());
			if(usr == 0L) {
				sprintf(ERR, "No user %s!\n", IWin->userEdit->text());
				if(dialog != 0L)
            	delete dialog;
				dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
            m = dialog->exec();
          	delete dialog;
           	delete IWin;
         	dialog = NULL;
           	IWin = NULL;
				return;
			}
			if(grp == 0L) {
				sprintf(ERR, "No group %s!\n", IWin->groupEdit->text());
				if(dialog != 0L)
           		delete dialog;
				dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
				m = dialog->exec();						
          	delete dialog;
           	delete IWin;
           	dialog = NULL;
           	IWin = NULL;
				return;
			}
			if(chown(myFullPath, usr->pw_uid, grp->gr_gid) != 0) {
				sprintf(ERR, "Could not change owner/group\n");
				if(dialog != 0L)
            	delete dialog;
				dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
				m = dialog->exec();						
            delete dialog;
      		delete IWin;
       		dialog = NULL;
      		IWin = NULL;
				return;
			} 
			updateMyData();
		}
	}
	
	if(myType == FIL)	{	
		if(IWin->mustresetedit) {
			((mfinderapp *)qApp)->unsetEditorOfFileAtPath(myFullPath);
			strcpy(myEditor, ((mfinderapp *)qApp)->editorOfFileAtPath(myFullPath));
		} else {
			if(strcmp(IWin->neweditorfield->text(), 
						((mfinderapp *)qApp)->editorOfFileAtPath(myFullPath)) != 0) {
				if(((mfinderapp*)qApp)->validateNewEditor((char *)IWin->neweditorfield->text())) {
					((mfinderapp *)qApp)->setEditorOfFileAtPath(myFullPath, 
															(char*)IWin->neweditorfield->text());		
					strcpy(myEditor, ((mfinderapp *)qApp)->editorOfFileAtPath(myFullPath));
				} else {
					sprintf(ERR, "You can't choose an editor\n"
									 "that is not in the apps list!\n"
									 "You can add an app to the apps list from\n"
									 "the Special->Configure->Programs menu\n");
					if(dialog != 0L)
            		delete dialog;
					dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
					m = dialog->exec();						
            	delete dialog;
      			delete IWin;
       			dialog = NULL;
      			IWin = NULL;
					return;
				}
			}
		}
	}
	if(IWin->mustreseticon) {
		havecustomicon = FALSE;
		((mfinderapp *)qApp)->unsetCustomIconOfFileAtPath(myFullPath);
		updateMyIcon();
		if(ismydeskobj) {
			mydeskobj->havecustomicon = FALSE;
			mydeskobj->updateMyIcon();
		}
	} else {
		if(IWin->mustchangeicon) {
			((mfinderapp *)qApp)->setCustomIconOfFileAtPath(myFullPath, IWin->newiconname);
			havecustomicon = TRUE;
			updateMyIcon();
			if(ismydeskobj) {
				mydeskobj->havecustomicon = TRUE;
				mydeskobj->updateMyIcon();
			}
		}
	}
	
	delete IWin;
	IWin = NULL;
}

void FileObj::updateMyData()
{
	struct stat listStat;
	char confdir[MAX_PATH_LEN] = "";
	char *pref, *suff, *token, *delims = ".";
	char fname[MAX_NAME_LEN] = "";
	FILE *fAppData;

	sprintf(confdir, "%s/GNUstep/Library/wmfinder", getenv("HOME"));
	lstat(myFullPath, &listStat);

	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			myType = FILEXE;
		else
			myType = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		myType = DR;
	if(S_ISLNK(listStat.st_mode)) {
		readlink(myFullPath, myLinkTo, MAX_PATH_LEN);
		if(chdir(myFullPath) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				myType = FILEXELNK;
			} else {
				myType = FILLNK;
			}
		} else {
			chdir(confdir);
			myType = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		myType = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		myType = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		myType = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		myType = SCKT;		
		
	if(isOpen)
		UnselOpen(TRUE);
	else
		Unselect(TRUE);

	isApp = FALSE;
		
	if(myType == FIL)	{
		strcpy(myEditor, ((mfinderapp *)qApp)->editorOfFileAtPath(myFullPath));

		if(strcmp(myPath, appsPath) == 0) {
			sprintf(fname, "%s", myName);
			token = strtok(fname, delims);
			pref = token;
			while(token != NULL) {
				suff = token;
				token = strtok(NULL, delims);
			}
			if(strcmp(suff, "app") == 0) {
				isApp = TRUE;
				fAppData = fopen(myFullPath, "r");
				fscanf(fAppData, "%s", myAppName);
				fclose(fAppData);
			}
		}
	}
	DeskObj *d = ((mfinderapp *)qApp)->deskObjectAtPath(myFullPath);
	if(d) {
		setMyDeskObj(d);
		d->setMyRefObj(this);
	} else {
		ismydeskobj = FALSE;
	}
}

void FileObj::updateMyIcon()
{
	Unselect(TRUE);
}

void FileObj::setMyDeskObj(DeskObj *dobj)
{
	mydeskobj = NULL;
	mydeskobj = dobj;
	ismydeskobj = TRUE;
}

void FileObj::unsetMyDeskObj()
{
	mydeskobj = NULL;
	ismydeskobj = FALSE;
}

void FileObj::Select(bool fromlocal)
{
	isSelect = TRUE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, SELECT, havecustomicon));
		label->setText(myName);
		label->raise();
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																	 TRUE, SELECT, havecustomicon));
	}
	label->setPalette(pinv);
	update();
	myobjsframe->unselectOtherFileObjects(myName);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Select(FALSE);
		}
	}	
}

void FileObj::Unselect(bool fromlocal)
{
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, UNSELECT, havecustomicon));
		label->setText(cutFileLabelText(myName));
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 TRUE, UNSELECT, havecustomicon));
	}
	label->setPalette(pnorm);
	update();
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Unselect(FALSE);
		}
	}		
}

void FileObj::SelOpen(bool fromlocal) 
{
	isOpenUnsl = FALSE;
	isOpen = TRUE;
	isSelect = TRUE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, SELOPEN, havecustomicon));
		label->setText(myName);
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 TRUE, SELOPEN, havecustomicon));
	}
	label->setPalette(pinv);
	label->raise();
	update();
	myobjsframe->unselectOtherFileObjects(myName);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->SelOpen(FALSE);
		}
	}			
}

void FileObj::UnselOpen(bool fromlocal)
{
	isOpenUnsl = TRUE;
	isOpen = TRUE;
	isSelect = FALSE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																	 FALSE, OPEN, havecustomicon));
		label->setText(cutFileLabelText(myName));
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																	 TRUE, OPEN, havecustomicon));
	}
	label->setPalette(pnorm);
	update();
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->UnselOpen(FALSE);
		}
	}				
}

void FileObj::Open(QMouseEvent *e)
{
	if(!isApp) {
		if(e->state() & ControlButton)
			((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, TRUE);
		else
			((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, FALSE);
	} else {
		((mfinderapp *)qApp)->openAppWithFile(myAppName, "", FALSE);
	}
}

void FileObj::Close(bool fromlocal)
{
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isSelect = TRUE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, SELECT, havecustomicon));
		label->setText(myName);
		label->raise();
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																	 TRUE, SELECT, havecustomicon));
	}
	label->setPalette(pinv);
	update();
	myobjsframe->unselectOtherFileObjects(myName);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Close(FALSE);
		}
	}					
}

void FileObj::openFromPopUp()
{
	((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, FALSE);		
}

void FileObj::renameFromPopUp()
{
	((mfinderapp *)qApp)->renameFileObjects(myobjsframe->myfwin);
}

void FileObj::infoFromPopUp()
{
	((mfinderapp *)qApp)->InfoObj(myobjsframe->myfwin);
}

void FileObj::editFromPopUp()
{
	((mfinderapp *)qApp)->Edit(myobjsframe->myfwin);
}

void FileObj::openWithFromPopUp()
{
	((mfinderapp *)qApp)->OpenWith(myobjsframe->myfwin);
}

void FileObj::duplicateFromPopUp()
{
	((mfinderapp *)qApp)->duplicateFileObjects(myobjsframe->myfwin);
}

void FileObj::deleteFromPopUp()
{
	((mfinderapp *)qApp)->removeFileObjects(myobjsframe->myfwin);
}

void FileObj::adjustLabelPosition()
{
	if(viewtype == BYICONS) {
		if(label->width() < width())
			label->move(x() + ((width() - label->width()) / 2), y() + 50);
		else
			label->move(x() - ((label->width() - width()) / 2), y() + 50);
	} else {
		label->move(x() + width() + 5, y());
	}
}

char *FileObj::cutFileLabelText(char *labelname)
{
	int w = 0, cw = 0, i = 0;
	int maxlabelw = 60;

	sprintf(cutname, "");
	QFontMetrics fm = QFontMetrics(font);
	QFontMetrics fmm = QFontMetrics(font);
	w = fm.boundingRect(labelname, strlen(labelname)).width();
	if(w > maxlabelw) {
		i = 0;
		while(cw <= maxlabelw - 3) {
			cutname[i] = labelname[i];
			cutname[i+1] = '\0';
			cw = fmm.boundingRect(cutname, strlen(cutname)).width();
			i++;
		}
		cutname[i] = '\0';
		if(strcmp(labelname, cutname) != 0) {
			if((strlen(labelname) - strlen(cutname)) < 3) {
				strncpy(cutname, labelname, 256);
			} else {
				cutname[i] = '.';
				cutname[i + 1] = '.';
				cutname[i + 2] = '.';
				cutname[i + 3] = '\0';
			}
		} 
	} else {
		strncpy(cutname, labelname, 256);
	}
	
	return cutname;
}

void FileObj::moveEvent(QMoveEvent *e)
{
	adjustLabelPosition();
}

void FileObj::mousePressEvent(QMouseEvent *e)
{
	dragdelay = 0;

	if(e->button() == RightButton) {
		myobjsframe->isShiftClick = FALSE;
		if(!popupMenu)
     		popupMenu = createPopup();
		popupMenu->popup(mapToGlobal(e->pos()));
	}

	if(e->state() & ShiftButton) {
		myobjsframe->isShiftClick = TRUE;
		if(isSelect) {
			if(isOpen) UnselOpen(true);
			else Unselect(true);
			return;
		} else {
			if(isOpen) SelOpen(true);
			else Select(true);
		}
	} else {
		myobjsframe->isShiftClick = FALSE;
		if(isOpenUnsl)
			SelOpen(true);
		if(isSelect) {

		} else {
			Select(true);
		}
	}
}

void FileObj::mouseDoubleClickEvent(QMouseEvent *e)
{
	dragdelay = 0;
	Open(e);
}

void FileObj::mouseMoveEvent(QMouseEvent *e)
{
	QStrList urls;
	QPixmap dragpix;

	dragdelay++;
	if(dragdelay >= 2) {
		urls = myobjsframe->makeUrlList();
		if(urls.count() == 1)
			dragpix = prepDragPix();
		else
			dragpix = ((mfinderapp *)qApp)->dragIconForFiles(urls.count());			
		myobjsframe->isShiftClick = FALSE;
		myobjsframe->unselectAllFileObjects();
		dropopenapp = isPlainFileAtPath(urls.at(0)) ? TRUE : FALSE;
		urlDragObj = new QUrlDrag(urls, this, "udrag");
		urlDragObj->setPixmap(dragpix, QPoint(0, 0));
		((mfinderapp *)qApp)->fileListDragObj = NULL;
		((mfinderapp *)qApp)->deskDragObj = NULL;
		((mfinderapp *)qApp)->fileDragObj = this;
		((mfinderapp *)qApp)->isdragcopying = FALSE;
		if(e->state() & ControlButton)
			((mfinderapp *)qApp)->isdragcopying = TRUE;
		((mfinderapp *)qApp)->xdnddone = FALSE;
		myobjsframe->objtokeep = this;
		
		urlDragObj->drag();
		
		if(!((mfinderapp *)qApp)->xdnddone) {
			myobjsframe->objtokeep = NULL;
			((mfinderapp *)qApp)->offixDrop(this);
		} else {
			myobjsframe->objtokeep = NULL;
			if(!((mfinderapp *)qApp)->isdragcopying && ((mfinderapp *)qApp)->dndok)
				myobjsframe->removeAnObject(this);
		}	
	}
}

void FileObj::dragEnterEvent(QDragEnterEvent *e)
{
	if(QUrlDrag::canDecode(e)) {
		if(isDir) {
			e->accept();
			myobjsframe->isShiftClick = FALSE;
			if(isOpenUnsl)
				SelOpen(true);
			if(isSelect) {

			} else {
				Select(true);
			}
			update();
		}
		if(isApp && dropopenapp) {
			e->accept();
			myobjsframe->isShiftClick = FALSE;
			if(isOpenUnsl)
				SelOpen(true);
			if(isSelect) {

			} else {
				Select(true);
			}
			update();
		}
	}
}

void FileObj::dragLeaveEvent(QDragLeaveEvent *e)
{
	if(isSelect) {
		if(isOpen)
			UnselOpen(true);
		else
			Unselect(true);
	}
}

void FileObj::dropEvent(QDropEvent *e)
{
	QStrList droplist;

	if(isDir) {
		e->accept();
		QUrlDrag::decode(e, droplist);
		((mfinderapp *)qApp)->moveOrCopy(droplist, myFullPath, this);
	} else if(isApp && dropopenapp) {
		e->accept();
		QUrlDrag::decode(e, droplist);
		((mfinderapp *)qApp)->openAppWithFile(myAppName, droplist.at(0), TRUE);
	}
}

QPixmap FileObj::prepDragPix()
{
	QPixmap dragpix;
	int labelw, labelh, icnw, icnh;

	icnw = icona.width() + 2;
	icnh = icona.height() + 2;
	labelw = label->width() + 2;
	labelh = label->height() + 2;
	if(viewtype == BYICONS) {
		if(labelw < icnw) {
			dragpix = QPixmap(icnw, 50 + labelh);
 			dragpix.fill(color1);
			bitBlt(&dragpix, 1, 1, &icona);
			bitBlt(&dragpix, (icnw - labelw) / 2, 50, label);
		} else {
			dragpix = QPixmap(labelw, 50 + labelh);
 			dragpix.fill(color1);
			bitBlt(&dragpix, (labelw - icnw) / 2, 1, &icona);
			bitBlt(&dragpix, 1, 50, label);
		}
	} else {
		dragpix = QPixmap(icnw + 5 + labelw, icnh);
 		dragpix.fill(color1);
		bitBlt(&dragpix, 1, 1, &icona);
		bitBlt(&dragpix, icnw + 5, 1, label);
	}
	dragpix.setMask(dragpix.createHeuristicMask());	

	return dragpix;
}

void FileObj::drawSelRect(QRect selrect, bool mustdraw)
{
	QRect oldrect;
	
	if(mustdraw) {
		mustdrawselrect = TRUE;
		
		oldrect = r1o;
		r1o = QRect(selrect.topLeft(), selrect.topRight());
		if(oldrect.topLeft().y() > 1)
			update(oldrect);
		if(r1o.topLeft().y() > 1)
			update(r1o);
		
		oldrect = r2o;
		r2o = QRect(selrect.bottomLeft(), selrect.bottomRight());
		if(oldrect.topLeft().y() < height() - 1)
			update(oldrect);
		if(r2o.topLeft().y() < height() - 1)
			update(r2o);

		oldrect = r1v;
		r1v = QRect(selrect.topLeft(), selrect.bottomLeft());
		if(oldrect.topLeft().x() > 1)
			update(oldrect);
		if(r1v.topLeft().x() > 1)
			update(r1v);

		oldrect = r2v;
		r2v = QRect(selrect.topRight(), selrect.bottomRight());
		if(oldrect.topLeft().x() < width() - 1)
			update(oldrect);
		if(r2v.topLeft().x() < width() - 1)
			update(r2v);
		
	} else {
		if(mustdrawselrect) {
			mustdrawselrect = FALSE;
			update(r1o);
			update(r2o);
			update(r1v);
			update(r2v);
		}
	}
}

void FileObj::paintEvent(QPaintEvent *)
{
	QPainter p;
	
	p.begin(this);
	p.drawPixmap(0, 0, icona);
	if(mustdrawselrect) {
		if(r1o.topLeft().y() > 1)
			p.drawRect(r1o);
		if(r2o.topLeft().y() < height() - 1)
			p.drawRect(r2o);
		if(r1v.topLeft().x() > 1)
			p.drawRect(r1v);
		if(r2v.topLeft().x() < width() - 1)
			p.drawRect(r2v);
	}
	p.end();
}

bool FileObj::eventFilter(QObject *, QEvent *e)
{
   if(e->type() == Event_MouseButtonPress 
			|| e->type() == Event_MouseButtonDblClick) {
		this->event(e);
		return TRUE;
  }
  return FALSE;
}



