#ifndef ___mfinderapp___
#define ___mfinderapp___

#include "common.h"
#include "dialogs.h"
#include "mfinderwin.h"
#include "fileobj.h"
#include "filelistobj.h"
#include "deskobj.h"
#include "objsframe.h"
#include "objslistframe.h"
#include "configure.h"
#include "findwin.h"

#ifdef __Linux__
	#include "procswin.h"
#endif

#include <qapp.h>
#include <qsignal.h>
#include <qdict.h>
#include <qlist.h> 
#include <qdatastream.h> 
#include <qfile.h>
#include <qpixmapcache.h> 
#include <qprogressbar.h> 
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qradiobutton.h> 
#include <qtimer.h> 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <errno.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sys/param.h>
#include <ctype.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xmu/WinUtil.h>
	
typedef struct {
	int uid, gid;
	char *name;
  	char home[MAX_PATH_LEN];
  	char shell[MAX_PATH_LEN];
  	char hostname[128];
  	mode_t umask;
} UserInfo;	
	
struct fileMetaData {
	char filepath[MAX_PATH_LEN];
	bool isopenlocal;
	bool hascustomicn;
	char custicnname[MAX_NAME_LEN];
	bool hascustomedit;
	char custedit[128];
};

struct appMetaData {
	char name[MAX_NAME_LEN];
	char classHintName[32];
	char classHintClass[32];
	bool warnifopen;
	int maxwait;
	char keyopen[2];
	bool kctrl;
	bool kalt;
	bool wmfaware;
};

struct runningApp {
	char appname[MAX_NAME_LEN];
	int pid;
};

struct processMetaData {
	appMetaData *appmdata;
	char appname[MAX_NAME_LEN];
	char cmdline[MAX_PATH_LEN];
	char openfilepath[MAX_PATH_LEN];
	char openfilename[MAX_NAME_LEN];
	int pid;
	time_t starttime;
	time_t elapsedtime;
	Window win;
	bool waitappwin;
	bool ungrabbed;
};

struct dictEntry {
	char key[MAX_PATH_LEN];
	char value[128];
};

// -- This is here because uses the struct "appMetaData" which is defined here
class ConfXProgramWin : public QWidget
{
	Q_OBJECT
public:

	ConfXProgramWin(QWidget *parent=0, const char *name=0, 
 												QList<char> sufflist=QList<char>(), 
 												QList<char> appnameslist=QList<char>(),
									QList<appMetaData> appmdlist=QList<appMetaData>());

	~ConfXProgramWin();

	QList<char> keyslist; 
	QList<char> appslist; 
	QList<char> tpmlist; 
	QList<appMetaData> appsmdlist; 

	Dialog *dialog;

	QLabel* titlebig;
	
	QLabel* expSuff;
	QListBox* suffslistbox;
	QLineEdit* linEditSuffix;	
	QLabel* expApp;
	QListBox* progslistbox;
	QLineEdit* linEditProg;
	QPushButton* buttChngProg;
	QPushButton* buttRemoveSuff;
	
	QLineEdit* linEditAddSuffix;
	QPushButton* buttAddSuff;
	
	QLabel* expAddApp;
	QLabel* expAppName;
	QLineEdit* leAppName;
	QLabel* expAppClHiName;
	QLineEdit* leAppClHiName;
	QLabel* expAppClHiClass;
	QLineEdit* leAppClHiClass;
	QLabel* expAppMaxWait;
	QLineEdit* leAppMaxWait;
	QLabel* expWarnIfOp;
	QCheckBox* ckWarnIfOp;	
	
	QLabel* expModKey;
	QLineEdit* leModKey;	
	QButtonGroup* bgrModKey;
	QRadioButton* ckCtrlKey;	
	QRadioButton* ckAltKey;	
	QCheckBox* ckWMFAware;	
	
	QPushButton* buttModifyProg;
	QPushButton* buttRemoveProg;
	QPushButton* buttClearProgForm;
	QPushButton* buttAddProg;	
	
	QLabel* explDefEd;
	QLineEdit* linEditEditor;
	
	QLabel* explXTerm;
	QLineEdit* linEditXTerm;

	QPushButton* butOk;
	QPushButton* butCancel;

public slots:
	void Close();
	void showSuffInEdField(const char*);
	void showAppNameInEdField(const char*);
	void changeAppForSuff();
	void removeSuff();
	void addSuff();

	void modifyApp();
	void removeApp();
	void clearAppForm();	
	void addApp();

	void abilitateOpenKeyCks(bool);
	void setWMFAware(bool);
		
protected:
	void paintEvent(QPaintEvent *);
	void resizeEvent(QResizeEvent *);

};
// ----------------------------------------------------------------------
	
class mfinderapp : public QApplication
{
	Q_OBJECT
public: 
   mfinderapp(int &argc, char** argv, const char *cln=NULL, 
											const unsigned char* icondata=NULL);

	QPixmap iconForFile(char *, char *, int, bool, int, bool);
	QPixmap dragIconForFiles(int);
	void unselectFileObjectsOfOtherWins(char *);
	void unselectAllObjNotDsObj(DeskObj *);
	void selectFileObjectFatherOfWindow(char *, bool);
	void openSelectedObjectsOfWin(MFinderWin *, bool);
	void openSelectedObjectsOfFileListObject(FileListObj *, bool);
	void openAtPath(char*, char*, int);
	MFinderWin* winAtPath(char *);
	FileObj* fileObjAtPath(char *);
	FileListObj* fileListObjAtPath(char *);
	void propagatePathChangeInWins(char *, char *);
	void ExecComm(char *, char *);
	void fileopsErr(char *, char *);
	void moveOrCopy(QStrList, char*, QWidget*);
	void newFileObject(MFinderWin*, bool);
	void removeFileObjects(MFinderWin*);	
	void renameFileObjects(MFinderWin*);	
	void duplicateFileObjects(MFinderWin*);	
	void openAppWithFile(char *, char *, bool);
	void Edit(MFinderWin*);	
	void OpenWith(MFinderWin*);	
	void StartXTerm(MFinderWin*);	
	void InfoObj(MFinderWin*);	
	bool validateNewEditor(char *);
	void findFile(char*);
	void doFind(char*);
	void offixDrop(QWidget*);		
	void makeDeskObjFromDrop(char*, int, int);
	void makeDeskObjFromDictEntry(char*);
	void removeDeskObj(DeskObj*);
	DeskObj* deskObjectAtPath(char *);
	bool x11EventFilter(XEvent *);
		
	QList<MFinderWin> winlist; 
	QList<DeskObj> deskobjlist; 

	MFinderWin *focusWin;
	Dialog *dialog;
	void centreDialogOnScreen(QWidget *);
	ConfBaseIcnWin *baseIconWin;
	ConfFilesIcnWin *filesIconWin;
	ConfXProgramWin *xprogramWin;
	
	WMFInfo *wmfinfo;
	#ifdef __Linux__
		ProcsWin *procwin;
	#endif

	bool isdragcopying;
	bool deskobjmoving;
	bool xdnddone;
	bool dndok;
	bool isfinding;
	int xappnum;
	FileObj *fileDragObj;
	FileListObj *fileListDragObj;
	DeskObj *deskDragObj;
	FindWin *findwin;
	
	Atom DndProtocol, DndSelection, WMFOpenAtom;
	Display *dspl;	
	int appScreen;			// X11 screen number
	Window appRootWin;	// X11 root window
	// -------------------- Offix DnD -------------------------------
	XButtonEvent	StartEvent;		/* event that started the drag */
	int				DragPrecision;	/* minimum dx,dy to start drag */
	int				Dragging;		/* Drag state flag */
	bool				DataOK;			/* Non-zero if data registered */
	Atom				WM_STATE;		/* Needed for icon stuff */
	Window			Target;			/* Drop window */
	Widget			MainWidget;		/* Main widget of application	*/
	int				DataType;		/* Current drag data type */
	XtEventHandler	OtherDrop;		/* Drop on non-reg app widget */
	XtEventHandler	RootDrop;		/* How to handle root drops */
	XtEventHandler	IconDrop;		/* How to handle icon drops */
	int				RootFlag;		/* Non-zero if dropped on root */
	long           LastDropVersion;/* Last drop protocol version */
	// --------------------------------------------------------------

	QTimer *timer;

	char Editor[128];
	char Xterm[255];
	char findstartpath[MAX_PATH_LEN];

	QList<dictEntry> xAppDict; 					// programs
	void setXAppDictEntryForKey(char*, char*);
	char *getXAppDictEntryForKey(char*);
	QList<dictEntry> wGeomDict;					// windows geometry
	char *rectOfWinAtPath(char*);
	QList<char> wOpenDict;							// windows to open
	QList<char> deskObjsDict;						// desk objects
	void addDeskObjToDict(DeskObj*);
	void removeDeskObjFromDict(char*);
	QList<dictEntry> icnReqDict;					// required icons
	void setIcnReqDictEntryForKey(char*, char*);
	char *getIcnReqDictEntryForKey(char*);
	QList<dictEntry> icnDict;						// icons
	void setIcnDictEntryForKey(char*, char*);
	char *getIcnDictEntryForKey(char*);
	QList<fileMetaData> filesDict;				// objects custom data
	char *editorOfFileAtPath(char*);
	bool hasCustomEditorFileAtPath(char*);
	void setEditorOfFileAtPath(char*, char*);
	void unsetEditorOfFileAtPath(char*);
	bool hasCustomIconFileAtPath(char*);
	char *customIconNameOfFileAtPath(char*);
	void setCustomIconOfFileAtPath(char*, char*);
	void unsetCustomIconOfFileAtPath(char*);
	void setIsOpenLocalOfFileAtPath(char*);
	void unsetIsOpenLocalOfFileAtPath(char*);
	bool isOpenLocalOfFileAtPath(char*);
	QList<processMetaData> processDict;
	QList<appMetaData> appsDict;
	QList<runningApp> runappsDict;
		
signals:
	
public slots:
	void processPendingData();
	void DirOpen(char *);
	void DirOpenFromFindFileWin(char *);
	void openDirAt();
	void DirClose(char *);
	void configureBaseIcons();
	void configureFilesIcons();
	void configureXPrograms();
	void makeNewReqIconList();
	void makeNewIconList();
	void makeNewXAppList();
	void parseFindQuery();
	void stopFind();
	void showInfo();
	
	#ifdef __Linux__
		void openProcessManager();
	#endif
	
	void writeDefaults();
	void quit();
	
protected:	
   char *appClassName;
   QPixmap *icon;
	
private: 
	UserInfo user;
	QFont font;
	
	StrtScrn *strscrn, *strscrnft;
	
	char ERR[512];
	ChooseAnIcnDlg *chanicndlg;
	
};

#endif
