#ifdef __Linux__

#include <sys/types.h> 
#include <signal.h> 
#include <unistd.h> 
#include <errno.h> 
#include "procsframe.h"
#include "procicn.h"

int compid(const void *a, const void *b)
{
	return(*((int *)a) - *((int *)b));	
}

ProcsFrame::ProcsFrame(QWidget *parent, const char *name, ProcsCntrls *contr)
			  : QWidget(parent, name)
{
	QColor x = QColor(230, 230, 230);
	QColorGroup CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, x);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));

	this->contr = contr;

	ImIcn = QImage(proc_xpm);
	ImusrIcn = QImage(user_xpm);
	ImttyIcn = QImage(monitor_xpm);
	ImcmdIcn = QImage(pencil_xpm);

	Icn = QPixmap();
	Icn.convertFromImage(ImIcn);
	usrIcn = QPixmap();
	usrIcn.convertFromImage(ImusrIcn);
	ttyIcn = QPixmap();
	ttyIcn.convertFromImage(ImttyIcn);
	cmdIcn = QPixmap();
	cmdIcn.convertFromImage(ImcmdIcn);

	IcnDeact = QPixmap();
	IcnDeact = makeTransIcn(ImIcn);
	IcnDeact.setMask(Icn.createHeuristicMask());	
	usrIcnDeact = QPixmap();
	usrIcnDeact = makeTransIcn(ImusrIcn);
	usrIcnDeact.setMask(usrIcn.createHeuristicMask());	
	ttyIcnDeact = QPixmap();
	ttyIcnDeact = makeTransIcn(ImttyIcn);
	ttyIcnDeact.setMask(ttyIcn.createHeuristicMask());	
	cmdIcnDeact = QPixmap();
	cmdIcnDeact = makeTransIcn(ImcmdIcn);
	cmdIcnDeact.setMask(cmdIcn.createHeuristicMask());	

	probjnum = 0;
	pnumtmp = 0;
	
	readprocs();
	sprintf(currusr, "root");
	for(int i = 0; i < contr->cmbUsers->count(); i++)
		if(strcmp(contr->cmbUsers->text(i), currusr) == 0)
			contr->cmbUsers->setCurrentItem(i);

	ShowUnUsrProcs(currusr);			
}

void ProcsFrame::readprocs()
{
	struct dirent **namelist;
	struct stat listStat;
	char path[64] = "";
	int n = 0, m = 0;	
	bool found;
		
	m = scandir("/proc", &namelist, 0, alphasort);
	for(n = 0; n < m; n++) {
		if(*namelist[n]->d_name > '0' && *namelist[n]->d_name <= '9') {
			sprintf(path, "/proc/%s", namelist[n]->d_name);
			if(stat(path, &listStat) != -1) {
				procobj[probjnum] = new ProcObj(this, "pobj", path);
				connect(procobj[probjnum], SIGNAL(clickonme(int)), 
							this, SLOT(DeselOthers(int)));
				probjnum++;
			}
		}
	}

	for(int i = 0; i < probjnum; i++)
		procspids[i] = procobj[i]->pid;
	
	qsort(procspids, probjnum, sizeof(int), compid);
	
	pnumtmp = 0;
	for(int i = 0; i < probjnum; i++) {
		for(int j = 0; j < probjnum; j++) {
			if(procspids[i] == procobj[j]->pid) {
				pobjTmp[pnumtmp] = procobj[j];
				pnumtmp++;
			}	
		}
	}
	for(int i = 0; i < probjnum; i++) {
		procobj[i] = pobjTmp[i];
		procobj[i]->id = i;
	}

	for(int i = 0; i < probjnum; i++) {
		for(int j = 0; j < probjnum; j++) {
			if(procobj[j]->ppid == procobj[i]->pid) {
				procobj[j]->parentid = i;
				procobj[j]->xShift = procobj[i]->xShift;
				procobj[j]->xShift++;
				procobj[i]->childsid[procobj[i]->nchilds] = j;
				procobj[i]->nchilds++;
			}
		}
	}
	
	pnumtmp = 0;
	for(int i = 0; i < probjnum; i++)
		makeProcTree(procobj[i]);

	for(int i = 0; i < probjnum; i++)
		procobj[i] = pobjTmp[i];

	for(int i = 0; i < probjnum; i++) {
		procobj[i]->nchilds = 0;
		for(int j = 0; j < probjnum; j++) {
			if(procobj[j]->ppid == procobj[i]->pid) {
				procobj[j]->parentid = i;
				procobj[i]->childsid[procobj[i]->nchilds] = j;
				procobj[i]->nchilds++;
			}
		}
	}

	for(int i = 0; i < contr->cmbUsers->count(); i++)
		contr->cmbUsers->removeItem(i); 
		
	for(int i = 0; i < probjnum; i++) {
		found = FALSE;
		for(int j = 0; j < contr->cmbUsers->count(); j++) {
			if(strcmp(procobj[i]->user, contr->cmbUsers->text(j)) == 0)
				found = TRUE;
		}
		if(!found) 
			contr->cmbUsers->insertItem(procobj[i]->user);
	}

	for(int i = 0; i < contr->cmbUsers->count(); i++)
		if(strcmp(contr->cmbUsers->text(i), currusr) == 0)
			contr->cmbUsers->setCurrentItem(i);

}

void ProcsFrame::makeProcTree(ProcObj *insObj)
{
	if(!insObj->intree) {
		pobjTmp[pnumtmp] = insObj;
		insObj->intree = TRUE;
		pnumtmp++;
		if(insObj->nchilds > 0) {
			for(int i = 0; i < insObj->nchilds; i++) {
				makeProcTree(procobj[insObj->childsid[i]]);
			}
		}
	}	
}

QPixmap ProcsFrame::makeTransIcn(QImage img)
{
	QPixmap pix;
	
	pix = QPixmap();
	img = img.convertDepth(32);
	pixmask = img.createHeuristicMask();
	for(int h = 0; h < img.width(); h++) {
		for(int j = 0; j < img.height(); j++) {
			QRgb pix = img.pixel(h, j);
			if(qRed(pixmask.pixel(h, j)) == 0) {
				int red = qRed(pix);
				int green = qGreen(pix);		
				int blue = qBlue(pix);
				if(red > 0) red = (int)(red/2);
				if((red + 120) <= 250) red = red + 120;
				else red = 250;
				if(green > 0) green = (int)(green/2);
				if((green + 120) <= 250) green = green + 120;
				else green = 250;
				if(blue > 0) blue = (int)(blue/2);
				if((blue + 120) <= 250) blue = blue + 120;
				else blue = 250;
				img.setPixel(h, j, qRgb(red, green, blue));
			}
		}
	}
	img = img.convertDepth(8);
	pix.convertFromImage(img);
	return pix;
}

void ProcsFrame::DeselOthers(int idobj)
{
	for(int i = 0; i < probjnum; i++)
		if(procobj[i]->id != idobj)
			if(procobj[i]->isSelect)
				procobj[i]->Unselect();
}

void ProcsFrame::reloadProcs()
{
	for(int i = 0; i < probjnum; i++) {
		procobj[i]->hide();
		procobj[i]->~ProcObj();
	}
	probjnum = 0;
	pnumtmp = 0;
	readprocs();
	QResizeEvent e(this->size(), this->size());
	this->event(&e);
	ShowUnUsrProcs(currusr);			
	repaint();
	for(int i = 0; i < probjnum; i++) {
		procobj[i]->show();
		
	}
}

void ProcsFrame::ShowUnUsrProcs(const char *usname)
{
	sprintf(currusr, "%s", usname);

	if(strcmp(usname, "root") != 0) {
		for(int i = 0; i < probjnum; i++)
			if(strcmp(procobj[i]->user, usname) != 0)
				procobj[i]->DeActivate();
			else
				procobj[i]->Activate();
	} else {
		for(int i = 0; i < probjnum; i++)
			procobj[i]->Activate();	
	}
}

void ProcsFrame::killProc()
{
	char mesg[256] = "", buf[32] = "";
	int i, j, kid = 0;
	bool found = FALSE;
	
	for(i = 0; i < probjnum; i++) {
		if(procobj[i]->isSelect) {
			found = TRUE;
			kid = i;
			for(j = 0; j < probjnum; j++) {
				if(procobj[j]->pid == procobj[i]->ppid) {
					sprintf(buf, "(%s)", procobj[j]->cmd);
				}
			}
			sprintf(mesg, "Are you sure that you want to kill\n%s\npid %i\
			- ppid %i %s\n", procobj[i]->cmd, procobj[i]->pid, procobj[i]->ppid, buf);
		}
	}
	if(found) {
		dialog = new Dialog(0L, "Kill", TRUE, 0, TRUE, TRUE, FALSE, TRUE, mesg);
		if(!dialog->exec()) {
			dialog->~Dialog();
			return;
		}			
		if(kill(procobj[kid]->pid, SIGQUIT) == -1) {
			switch(errno) {
				case EINVAL:
					sprintf(mesg, "An invalid signal was sent!\n");		
					break;
				case ESRCH:
					sprintf(mesg, "The pid, %i does not exist.\n\
					It might be already terminated or might be a zombie\n", procobj[kid]->pid);
					break;
				case EPERM:
					sprintf(mesg, "You have not permission for sending this signal to\n\
					%s (%i)\n", procobj[kid]->cmd, procobj[kid]->pid);	
					break;			
				default:
					sprintf(mesg, "System Error!\n");
					break;
			}
			dialog->~Dialog();
			dialog = new Dialog(0L, "Kill", TRUE, 0, FALSE, TRUE, FALSE, FALSE, mesg);
			dialog->exec();
		} else {
			sprintf(mesg, "The process with pid %i (%s)\nwas killed!\n", 
											procobj[kid]->pid, procobj[kid]->cmd);
			dialog->~Dialog();								
			dialog = new Dialog(0L, "Kill", TRUE, 0, FALSE, TRUE, FALSE, FALSE, mesg);
			int dex = dialog->exec();
			reloadProcs();
		}
	}
}

void ProcsFrame::resizeEvent(QResizeEvent *)
{
	int VObjs = 0;
	
	for(int i = 0; i < probjnum; i++) {
		procobj[i]->move(procobj[i]->xShift * 20, VObjs);
		procobj[i]->resize(width(), 15);
		VObjs = VObjs + 20;
	}
	emit resizeDone(VObjs + 40);
}

void ProcsFrame::closeEvent(QCloseEvent *e)
{
	for(int i = 0; i < probjnum; i++)
		procobj[i]->~ProcObj();

	QWidget::closeEvent(e);
}

void ProcsFrame::paintEvent(QPaintEvent *e)
{
	QPainter p;

	p.begin(this);

	for(int i = 0; i < probjnum; i++)
		if(procobj[i]->ppid != 0)
			p.drawLine(procobj[i]->pos().x() - 12, procobj[i]->pos().y() + 7, 
							procobj[i]->pos().x() - 4, procobj[i]->pos().y() + 7);

	for(int i = 0; i < probjnum; i++) {
		if(procobj[i]->nchilds > 0) {

			p.drawLine(procobj[i]->pos().x() + 7, 
					 	  procobj[i]->pos().y() + 18,
				  		  procobj[i]->pos().x() + 7, 
			procobj[procobj[i]->childsid[procobj[i]->nchilds - 1]]->pos().y() + 7);
	
		}
	}
	
	p.end();
}

#endif

